{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzDatum;
{ GeographicProjection.cpp
 Projection.cpp
 DatumConvertor.cpp}

interface

{uses buzGlobal; }
{uses SysUtils,Dialogs,Math;}

Procedure ConvertDatums( {dXTemp,dYTemp,DXdtemp,DYdtemp:Double);}
                    LongXCoord, LatYCoord:Double;
                    var XCoord, YCoord:Double);
Function convertFromWGS84(var lat, lon, height:Double{ DATUM to }):Boolean;
Function convertToWGS84(var  lat, lon, height:Double{ DATUM from }):Boolean;
Function getEllipsoidParameters( ELLIPSOIDe:String ):Boolean;
Function getDatumEllipsoid( DATUMd:Integer ):String;
Function getDatumParameters( DATUMd:Integer ):Boolean;
Procedure FillDatumEllipsoids;
{  static bool convertDatum( double& lat, double& lon, DATUM from, DATUM to );
  static ELLIPSOID getDatumEllipsoid( DATUM d );

  static bool convertToWGS84( double& lat, double& lon, double& height,
                              DATUM from );
  static bool convertFromWGS84( double& lat, double& lon, double& height,
                                DATUM to );
  static DatumParameters* getDatumParameters( DATUM d );
  static EllipsoidParameters* getEllipsoidParameters( ELLIPSOID e );
  /*static bool convertToECEF( double& lat, double& lon, double& height,
                               DATUM from );
  static bool convertFromECEF( double& lat, double& lon, double& height,
                               DATUM to );*/

  // Conversion parameters array
  static DatumParameters     d_datumParams[];
  static EllipsoidParameters d_ellipsoidParams[];
  }
implementation

uses buzGlobal;


TYPE
   DatumParameters=Record
    DATUMd_datum,
    ELLIPSOIDd_ellipsoid:String;
     d_deltaX,
     d_deltaY,
     d_deltaZ:double;
  end;

  // Ellipsoid info type
   EllipsoidParameters=Record
    ELLIPSOIDd_ellipsoid:String;
     d_deltaA, // equatorial radius shift from WGS84
     d_deltaF:double; // flattening shift from WGS84
  end;
var
WorkDatum :DatumParameters;
WorkEllipsoid:EllipsoidParameters;
// Define WGS84 constants
{#define PI 3.1415926535 }
WGS84_RADIUS:double= 6378137.0;
WGS84_FLATTENING:double= ( 1.0 / 298.257223563 );

// Initialize datum conversion parameters array
//DatumConvertor::DatumParameters DatumConvertor::
d_datumParams: array[0..23] of DatumParameters;
d_ellipsoidParams: array[0..13] of EllipsoidParameters;

                  {Already Radians}
Procedure ConvertDatums( {dXTemp,dYTemp,DXdtemp,DYdtemp:Double);}
                    LongXCoord, LatYCoord:Double;
                    var XCoord, YCoord:Double);
var  lat, lon, height:double;
Begin
  // XXX - Should I be getting the actual height for this location from a
  // table or something?  The height doesn't seem to effect the results much,
  // at least within the accuracy limits of USGS data.
  height := 0.0;
  lat:=LatYCoord;
  lon:=LongXCoord;
  // Convert the point to WGS84
  if ( inparm[18]<>23 {WGS_84 != from} )then
  begin
    if (not convertToWGS84(lat, lon, height{, from} ) )then
    begin
      DoProjError(2002,2002,1,1 ,inparm[18]);
      Exit;
    end;
  end;

  // Convert the point from WGS84
  if (outparm[18]<>23{ WGS_84 != to} )then
  begin
    if (not convertFromWGS84( lat, lon, height{, to} ) )then
    begin
      DoProjError(2002,2002,2,1 ,inparm[18]);
      Exit;
    end;
  end;
YCoord:=lat;
XCoord:=lon;
End;

Function convertFromWGS84(var  lat, lon, height:Double):Boolean;
var
z,deltaA,deltaF,fromA,fromF,fromES,bda,sLat,cLat,sLon,
cLon,dx,dy,dz,tmp,sqrtTmp,rn,rm,dLat,dLon,dHeight
:Double;
Begin
   { DATUM to
   DatumParameters=Record
    DATUMd_datum,
    ELLIPSOIDd_ellipsoid:String;
     d_deltaX,
     d_deltaY,
     d_deltaZ:double;}
   If (not getDatumParameters(Round(inparm[18])))
   then
   Begin
     DoProjError(2002,2002,3,1 ,inparm[18]);
     Result:=False;
     Exit;
   End;
   z := height;

  // Get the ellipsoid parameters
    { EllipsoidParameters=Record
    ELLIPSOIDd_ellipsoid:String;
     d_deltaA, // equatorial radius shift from WGS84
     d_deltaF:double; // flattening shift from WGS84}
   If (not getEllipsoidParameters( WorkDatum.ELLIPSOIDd_ellipsoid ))
     then
   Begin
     DoProjError(2002,2002,4,1 ,inparm[18]);
     Result:=False;
     Exit;
   End;

  // Initialize parameters
   deltaA := -WorkEllipsoid.d_deltaA;
   deltaF := -WorkEllipsoid.d_deltaF;
   fromA := WGS84_RADIUS;
   fromF := WGS84_FLATTENING;
   fromES := ( 2.0 - fromF ) * fromF;
   bda := 1.0 - fromF;
   sLat := sin( lat );
   cLat := cos( lat );
   sLon := sin( lon );
   cLon := cos( lon );
   dx := -WorkDatum.d_deltaX;
   dy := -WorkDatum.d_deltaY;
   dz := -WorkDatum.d_deltaZ;

  // Calculate the radius of curvature in prime veritcal
   tmp := 1.0 - fromES * sLat * sLat;
   sqrtTmp := sqrt( tmp );
   rn := fromA / sqrtTmp;

  // Calculate the radius of curvature in prime meridian
   rm := fromA * ( 1.0 - fromES ) / ( tmp * sqrtTmp );

  // Calculate the difference in latitude
   dLat := ( -dx * sLat * cLon - dy * sLat * sLon ) + dz * cLat;
  dLat := dLat +(( deltaA * rn * fromES * sLat * cLat ) / fromA);
  dLat := dLat +(deltaF * ( rm / bda + rn * bda ) * sLat * cLat);
  dLat := dLat /(rm + z);

  // Calculate the difference in longitude
   dLon := ( -dx * sLon + dy * cLon ) / ( cLat * ( rn + z ) );

  // Calculate the difference in height
   dHeight := dx * cLat * cLon + dy * cLat * sLon + dz * sLat -
                   ( deltaA * fromA ) / rn;
  dHeight := dHeight+(deltaF * bda * rn * sLat * sLat);

  lat := lat+dLat;
  lon := lon+dLon;
  height := height+dHeight;

Result:= true;
End;


Function convertToWGS84(var  lat, lon, height:Double):Boolean;
var
z,deltaA,deltaF,fromA,fromF,fromES,bda,sLat,cLat,sLon,
cLon,dx,dy,dz,tmp,sqrtTmp,rn,rm,dLat,dLon,dHeight
:Double;
Begin{ DATUM from }
  // Use z as zero for now
   z := height;
  // Get the datum parameters
   If (not getDatumParameters(Round(outparm[18])))
   then
   Begin
     DoProjError(2002,2002,5,1 ,inparm[18]);
     Result:=False;
     Exit;
   End;
  // Get the ellipsoid parameters
   If (not getEllipsoidParameters( WorkDatum.ELLIPSOIDd_ellipsoid ))
   then
   Begin
     DoProjError(2002,2002,6,1 ,inparm[18]);
     Result:=False;
     Exit;
   End;
  // Initialize parameters
   deltaA := WorkEllipsoid.d_deltaA;
   deltaF := WorkEllipsoid.d_deltaF;
   fromA := WGS84_RADIUS - deltaA;
   fromF := WGS84_FLATTENING - deltaF;
   fromES := ( 2.0 - fromF ) * fromF;
   bda := 1.0 - fromF;
   sLat := sin( lat );
   cLat := cos( lat );
   sLon := sin( lon );
   cLon := cos( lon );
   dx := WorkDatum.d_deltaX;
   dy := WorkDatum.d_deltaY;
   dz := WorkDatum.d_deltaZ;

  // Calculate the radius of curvature in prime veritcal
   tmp := 1.0 - fromES * sLat * sLat;
   sqrtTmp := sqrt( tmp );
   rn := fromA / sqrtTmp;

  // Calculate the radius of curvature in prime meridian
   rm := fromA * ( 1.0 - fromES ) / ( tmp * sqrtTmp );

  // Calculate the difference in latitude
   dLat := ( -dx * sLat * cLon - dy * sLat * sLon ) + dz * cLat;
  dLat := dLat+(( deltaA * rn * fromES * sLat * cLat ) / fromA);
  dLat := dLat+(deltaF * ( rm / bda + rn * bda ) * sLat * cLat);
  dLat := dLat/(rm + z);

  // Calculate the difference in longitude
   dLon := ( -dx * sLon + dy * cLon ) / ( cLat * ( rn + z ) );

  // Calculate the difference in height
   dHeight := dx * cLat * cLon + dy * cLat * sLon + dz * sLat -
                   ( deltaA * fromA ) / rn;
   dHeight :=dHeight + deltaF * bda * rn * sLat * sLat;

  lat := lat+dLat;
  lon := lon+dLon;
  height := height+dHeight;

Result:= true;
End;

(*inline bool
Projection::checkCoordinateRange( double latitude, double longitude ) const
{
    return ( ( latitude <= 90.0 && latitude >= -90.0 ) &&
             ( longitude <= 180.0 && longitude >= -180.0 ) );
}
*)

(*
/*
bool
DatumConvertor::convertToECEF( double& lat, double& lon, double& height,
                               DATUM from )
{
  // Get the conversion parameters
  DatumParameters* pParams = getConversionParameters( from );
  if ( NULL == pParams )
  {
     return false;
  }

  // Initialize parameters
  double deltaA = pParams->d_deltaA;
  double deltaF = pParams->d_deltaF;
  double fromA = WGS84_RADIUS - deltaA;
  double fromF = WGS84_FLATTENING - deltaF;
  double fromES = ( 2.0 - fromF ) * fromF;
  double polarRadius = fromA * ( 1.0 - fromF );
  double sLat = sin( lat );
  double cLat = cos( lat );
  double sLon = sin( lon );
  double cLon = cos( lon );
  double N = fromA / sqrt( 1 - fromES * sLat * sLat );

  lon = ( N + height ) * cLat * cLon + pParams->d_deltaX;
  lat = ( N + height ) * cLat * sLon + pParams->d_deltaY;
  height = ( N * ( 1.0 - fromES ) + height ) * sLat + pParams->d_deltaZ;

  return true;
}

bool
DatumConvertor::convertFromECEF( double& lat, double& lon, double& height,
                                 DATUM to )
{
  // Get the conversion parameters
  DatumParameters* pParams = getConversionParameters( to );
  if ( NULL == pParams )
  {
    return false;
  }

  lon -= pParams->d_deltaX;
  lat -= pParams->d_deltaY;
  height -= pParams->d_deltaZ;

  // Initialize parameters
  double deltaA = pParams->d_deltaA;
  double deltaF = pParams->d_deltaF;
  double toA = WGS84_RADIUS - deltaA;
  double toF = WGS84_FLATTENING - deltaF;
  double toES = ( 2.0 - toF ) * toF;
  double polarRadius = toA * ( 1.0 - toF );
  double p = sqrt( lat * lat + lon * lon );
  double theta = atan( height / p );
  double e2s = ( toA * toA - polarRadius * polarRadius )
    / ( polarRadius * polarRadius );

  double Y = lat;
  lat = atan( ( height + e2s * polarRadius * sin( theta ) * sin( theta )
    * sin( theta ) ) / ( p - toES * toA * cos( theta ) * cos( theta )
    * cos( theta ) ) );
  lon = atan2( Y, lon );

  double sLat = sin( lat );
  double N = toA / sqrt( 1 - toES * sLat * sLat );
  height = p / cos( lat ) - N;

  return true;
}*)

Function  getEllipsoidParameters( ELLIPSOIDe:String ):Boolean;
{d_ellipsoidParams: array[0..13] of EllipsoidParameters;}
var I:Integer;
Begin
  Result:=False;
  For I:=0 to 13 do
  Begin
    If (ELLIPSOIDe =d_ellipsoidParams[I].ELLIPSOIDd_ellipsoid ) then
    begin
      WorkEllipsoid.ELLIPSOIDd_ellipsoid:=
        d_ellipsoidParams[I].ELLIPSOIDd_ellipsoid;
      WorkEllipsoid.d_deltaA:=
        d_ellipsoidParams[I].d_deltaA;
      WorkEllipsoid.d_deltaF:=
        d_ellipsoidParams[I].d_deltaF;
      Result:=True;
    end;
  End;
End;

{ELLIPSOID DatumConvertor::}
Function getDatumEllipsoid( DATUMd:Integer ):String;
begin
  If getDatumParameters( DATUMd )then
    getDatumEllipsoid:=
      WorkDatum.ELLIPSOIDd_ellipsoid
    else
    getDatumEllipsoid:='';
end;

Function  getDatumParameters( DATUMd:Integer ):Boolean;
{d_datumParams: array[0..23] of DatumParameters;}
begin
  WorkDatum.DATUMd_datum:=d_datumParams[DATUMd].DATUMd_datum;
  WorkDatum.ELLIPSOIDd_ellipsoid:=d_datumParams[DATUMd].ELLIPSOIDd_ellipsoid;
  WorkDatum.d_deltaX:=d_datumParams[DATUMd].d_deltaX;
  WorkDatum.d_deltaY:=d_datumParams[DATUMd].d_deltaY;
  WorkDatum.d_deltaZ:=d_datumParams[DATUMd].d_deltaZ;
  Result:=True;
End;


Procedure FillDatumEllipsoids;
Begin
{d_datumParams: array[0..23] of DatumParameters;}
   d_datumParams[0].DATUMd_datum:='ADINDAN';
   d_datumParams[0].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_datumParams[0].d_deltaX:=-162;
   d_datumParams[0].d_deltaY:=-12;
   d_datumParams[0].d_deltaZ:= -206;

   d_datumParams[1].DATUMd_datum:='ARC1950';
   d_datumParams[1].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_datumParams[1].d_deltaX:=-143;
   d_datumParams[1].d_deltaY:=-90;
   d_datumParams[1].d_deltaZ:= -294;

   d_datumParams[2].DATUMd_datum:='ARC1960';
   d_datumParams[2].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_datumParams[2].d_deltaX:=-160;
   d_datumParams[2].d_deltaY:=-8;
   d_datumParams[2].d_deltaZ:= -300;

   d_datumParams[3].DATUMd_datum:='AUSTRALIAN_GEODETIC_1966';
   d_datumParams[3].ELLIPSOIDd_ellipsoid:='AUSTRALIAN_NATIONAL';
   d_datumParams[3].d_deltaX:=-133;
   d_datumParams[3].d_deltaY:=-48;
   d_datumParams[3].d_deltaZ:= -148;

   d_datumParams[4].DATUMd_datum:='AUSTRALIAN_GEODETIC_1984';
   d_datumParams[4].ELLIPSOIDd_ellipsoid:='AUSTRALIAN_NATIONAL';
   d_datumParams[4].d_deltaX:=-134;
   d_datumParams[4].d_deltaY:=-48;
   d_datumParams[4].d_deltaZ:= 149;

   d_datumParams[5].DATUMd_datum:='CAMP_AREA_ASTRO';
   d_datumParams[5].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[5].d_deltaX:=-104;
   d_datumParams[5].d_deltaY:=-129;
   d_datumParams[5].d_deltaZ:= 239;

   d_datumParams[6].DATUMd_datum:='CAPE';
   d_datumParams[6].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_datumParams[6].d_deltaX:=-136;
   d_datumParams[6].d_deltaY:=-108;
   d_datumParams[6].d_deltaZ:= -292;

   d_datumParams[7].DATUMd_datum:='EUROPEAN_DATUM_1950';
   d_datumParams[7].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[7].d_deltaX:=-87;
   d_datumParams[7].d_deltaY:=-98;
   d_datumParams[7].d_deltaZ:= -121;

   d_datumParams[8].DATUMd_datum:='EUROPEAN_DATUM_1979';
   d_datumParams[8].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[8].d_deltaX:=-86;
   d_datumParams[8].d_deltaY:=-98;
   d_datumParams[8].d_deltaZ:= -119;

   d_datumParams[9].DATUMd_datum:='GEODETIC_DATUM_1949';
   d_datumParams[9].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[9].d_deltaX:=84;
   d_datumParams[9].d_deltaY:=-22;
   d_datumParams[9].d_deltaZ:= 209;

   d_datumParams[10].DATUMd_datum:='HONG_KONG_1963';
   d_datumParams[10].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[10].d_deltaX:=-156;
   d_datumParams[10].d_deltaY:=-271;
   d_datumParams[10].d_deltaZ:= -189;

   d_datumParams[11].DATUMd_datum:='HU_TZU_SHAN';
   d_datumParams[11].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[11].d_deltaX:=-634;
   d_datumParams[11].d_deltaY:=-549;
   d_datumParams[11].d_deltaZ:= -201;

   d_datumParams[12].DATUMd_datum:='INDIAN';
   d_datumParams[12].ELLIPSOIDd_ellipsoid:='EVEREST';
   d_datumParams[12].d_deltaX:=289;
   d_datumParams[12].d_deltaY:=734;
   d_datumParams[12].d_deltaZ:= 257;

   d_datumParams[13].DATUMd_datum:='NAD27';
   d_datumParams[13].ELLIPSOIDd_ellipsoid:='CLARKE_1866';
   d_datumParams[13].d_deltaX:=-8;
   d_datumParams[13].d_deltaY:=160;
   d_datumParams[13].d_deltaZ:= 176;

   d_datumParams[14].DATUMd_datum:='NAD83';
   d_datumParams[14].ELLIPSOIDd_ellipsoid:='GRS_1980';
   d_datumParams[14].d_deltaX:=0;
   d_datumParams[14].d_deltaY:=0;
   d_datumParams[14].d_deltaZ:= 0;

   d_datumParams[15].DATUMd_datum:='OLD_HAWAIIAN_MEAN';
   d_datumParams[15].ELLIPSOIDd_ellipsoid:='CLARKE_1866';
   d_datumParams[15].d_deltaX:=89;
   d_datumParams[15].d_deltaY:=-279;
   d_datumParams[15].d_deltaZ:= -183;

   d_datumParams[16].DATUMd_datum:='OMAN';
   d_datumParams[16].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_datumParams[16].d_deltaX:=-346;
   d_datumParams[16].d_deltaY:=-1;
   d_datumParams[16].d_deltaZ:= 224;

   d_datumParams[17].DATUMd_datum:='ORDNANCE_SURVEY_1936';
   d_datumParams[17].ELLIPSOIDd_ellipsoid:='AIRY';
   d_datumParams[17].d_deltaX:=375;
   d_datumParams[17].d_deltaY:=-111;
   d_datumParams[17].d_deltaZ:= 431;

   d_datumParams[18].DATUMd_datum:='PUERTO_RICO';
   d_datumParams[18].ELLIPSOIDd_ellipsoid:='CLARKE_1866';
   d_datumParams[18].d_deltaX:=11;
   d_datumParams[18].d_deltaY:=72;
   d_datumParams[18].d_deltaZ:= -101;

   d_datumParams[19].DATUMd_datum:='PULKOVO_1942';
   d_datumParams[19].ELLIPSOIDd_ellipsoid:='KRASSOVSKY';
   d_datumParams[19].d_deltaX:=27;
   d_datumParams[19].d_deltaY:=-135;
   d_datumParams[19].d_deltaZ:= -89;

   d_datumParams[20].DATUMd_datum:='PROVISIONAL_S_AMERICAN_1956';
   d_datumParams[20].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_datumParams[20].d_deltaX:=-288;
   d_datumParams[20].d_deltaY:=175;
   d_datumParams[20].d_deltaZ:= -376;

   d_datumParams[21].DATUMd_datum:='TOKYO';
   d_datumParams[21].ELLIPSOIDd_ellipsoid:='BESSEL';
   d_datumParams[21].d_deltaX:=-128;
   d_datumParams[21].d_deltaY:=481;
   d_datumParams[21].d_deltaZ:= 664;

   d_datumParams[22].DATUMd_datum:='WGS_72';
   d_datumParams[22].ELLIPSOIDd_ellipsoid:='WGS_72_ELLIPSOID';
   d_datumParams[22].d_deltaX:=0;
   d_datumParams[22].d_deltaY:=0;
   d_datumParams[22].d_deltaZ:= -4.5;

   d_datumParams[23].DATUMd_datum:='WGS_84';
   d_datumParams[23].ELLIPSOIDd_ellipsoid:='WGS_84_ELLIPSOID';
   d_datumParams[23].d_deltaX:=0;
   d_datumParams[23].d_deltaY:=0;
   d_datumParams[23].d_deltaZ:= 0;

{d_ellipsoidParams: array[0..13] of EllipsoidParameters;}

   d_ellipsoidParams[0].ELLIPSOIDd_ellipsoid:='CLARKE_1866';
   d_ellipsoidParams[0].d_deltaA:=-69.4;
   d_ellipsoidParams[0].d_deltaF:=-0.000037264639;


   d_ellipsoidParams[1].ELLIPSOIDd_ellipsoid:='CLARKE_1880';
   d_ellipsoidParams[1].d_deltaA:=-112.145;
   d_ellipsoidParams[1].d_deltaF:=-0.000054750714;

   d_ellipsoidParams[2].ELLIPSOIDd_ellipsoid:='BESSEL';
   d_ellipsoidParams[2].d_deltaA:=739.845;
   d_ellipsoidParams[2].d_deltaF:=0.000010037483;

   d_ellipsoidParams[3].ELLIPSOIDd_ellipsoid:='INTERNATIONAL_1967';
   d_ellipsoidParams[3].d_deltaA:=-251.0;
   d_ellipsoidParams[3].d_deltaF:=-0.000014192702;

   d_ellipsoidParams[4].ELLIPSOIDd_ellipsoid:='WGS_72_ELLIPSOID';
   d_ellipsoidParams[4].d_deltaA:=2.0;
   d_ellipsoidParams[4].d_deltaF:=0.00000003121057;

   d_ellipsoidParams[5].ELLIPSOIDd_ellipsoid:='EVEREST';
   d_ellipsoidParams[5].d_deltaA:=860.655;
   d_ellipsoidParams[5].d_deltaF:=0.000028361368;

   d_ellipsoidParams[6].ELLIPSOIDd_ellipsoid:='GRS_1980';
   d_ellipsoidParams[6].d_deltaA:=0.0;
   d_ellipsoidParams[6].d_deltaF:=-0.000000000016;

   d_ellipsoidParams[7].ELLIPSOIDd_ellipsoid:='AIRY';
   d_ellipsoidParams[7].d_deltaA:=573.604;
   d_ellipsoidParams[7].d_deltaF:=0.000011960023;

   d_ellipsoidParams[8].ELLIPSOIDd_ellipsoid:='MODIFIED_EVEREST';
   d_ellipsoidParams[8].d_deltaA:=832.937;
   d_ellipsoidParams[8].d_deltaF:=0.000028361368;

   d_ellipsoidParams[9].ELLIPSOIDd_ellipsoid:='MODIFIED_AIRY';
   d_ellipsoidParams[9].d_deltaA:=796.811;
   d_ellipsoidParams[9].d_deltaF:=0.000011960023;

   d_ellipsoidParams[10].ELLIPSOIDd_ellipsoid:='WGS_84_ELLIPSOID';
   d_ellipsoidParams[10].d_deltaA:=0.0;
   d_ellipsoidParams[10].d_deltaF:=0.0;

   d_ellipsoidParams[11].ELLIPSOIDd_ellipsoid:='AUSTRALIAN_NATIONAL';
   d_ellipsoidParams[11].d_deltaA:=-23.0;
   d_ellipsoidParams[11].d_deltaF:=-0.000000081204;

   d_ellipsoidParams[12].ELLIPSOIDd_ellipsoid:='KRASSOVSKY';
   d_ellipsoidParams[12].d_deltaA:=-108;
   d_ellipsoidParams[12].d_deltaF:=0.000000480795;

   d_ellipsoidParams[13].ELLIPSOIDd_ellipsoid:='HOUGH';
   d_ellipsoidParams[13].d_deltaA:=-133;
   d_ellipsoidParams[13].d_deltaF:=-0.000014192702;
End;

end.
